# Generated by roxygen2: do not edit by hand

S3method(print,linf.csts)
S3method(summary,linf.csts)
export(asv.to.linf.csts)
export(collapse.rare)
export(expand.rare)
export(filter.asv)
export(latex.linf.csts)
export(linf.cells)
export(linf.csts)
export(linf.dcst.landmark.pipeline)
export(linf.feature.labels)
export(linf.hypercube.embedding)
export(linf.landmarks)
export(normalize.linf)
export(refine.linf.csts)
export(refine.linf.csts.iter)
export(transfer.dcsts)
importFrom(methods,as)
