% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linf.R
\name{dcst.view}
\alias{dcst.view}
\title{Select a stored dCST policy view}
\usage{
dcst.view(csts, view = c("absorb", "pure"))
}
\arguments{
\item{csts}{A \code{"linf.csts"} object produced by \code{\link{linf.csts}}
and optionally refined by \code{\link{refine.linf.csts}} /
\code{\link{refine.linf.csts.iter}}.}

\item{view}{Character. The stored policy view to activate: \code{"pure"} or
\code{"absorb"}.}
}
\value{
A \code{"linf.csts"} object using the requested view.
}
\description{
Returns a copy of a \code{"linf.csts"} object using either the stored
\code{"pure"} or \code{"absorb"} hierarchy. This does not recompute dCSTs.
}
