% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linf.R
\name{summary.linf.csts}
\alias{summary.linf.csts}
\title{Summarize dCST hierarchy statistics}
\usage{
\method{summary}{linf.csts}(object, ...)
}
\arguments{
\item{object}{A \code{"linf.csts"} object.}

\item{...}{Unused.}
}
\value{
Data frame with depth-wise statistics
}
\description{
Summarize dCST hierarchy statistics
}
