% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linf.R
\name{print.linf.csts}
\alias{print.linf.csts}
\title{Print method for \code{"linf.csts"}}
\usage{
\method{print}{linf.csts}(x, ...)
}
\arguments{
\item{x}{A \code{"linf.csts"} object.}

\item{...}{Unused.}
}
\value{
The input object, invisibly.
}
\description{
Print method for \code{"linf.csts"}
}
